==> prayer-1.0.12.tar.gz: prayer-1.0.12/prayer/cmd_passwd.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_passwd.c,v 1.2 2003/05/21 10:10:55 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
html_form_table_passwd_field(struct buffer *b,
                             char *desc, char *name, char *value,
                             unsigned long size)
{
    bputs(b, "<tr>" CRLF);

    if (desc && desc[0])
        bprintf(b, "<td>%s</td>", desc);

    bprintf(b, "<td><input name=\"%s\" type=\"password\" value=\"", name);
    if (value)
        html_quote_string(b, value);
    bprintf(b, "\" size=\"%lu\"></td>" CRLF, size);

    bputs(b, "</tr>" CRLF);
}

static void
generate_form(struct session *session, char *old, char *new, char *new2)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Title bar */
    html_start(session, b, "Change Password");
    html_banner_toolbar(session, b, "passwd", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Change Password</h2>" CRLF);

    bputs(b, "Restrictions on passwords:" CRLF);
    bputs(b, "<ul>" CRLF);
    bputs(b, "<li>Must be at least six characters in length</li>" CRLF);
    bputs(b,
          "<li>Only the first eight characters are significant</li>" CRLF);
    bputs(b,
          "<li>Must contain at least one letter and one non-letter</li>"
          CRLF);
    bputs(b, "</ul>" CRLF);
    if (config->accountd_passwd_delay > (2 * 60 * 60))
        bprintf(b,
                ("NB: It can take up to %lu hours for password changes "
                 "to take effect" CRLF),
                (config->accountd_passwd_delay / (60 * 60)));
    else if (config->accountd_passwd_delay > 120)
        bprintf(b,
                ("NB: It can take up to %lu minutes for password changes "
                 "to take effect" CRLF),
                (config->accountd_passwd_delay / 60));
    else if (config->accountd_passwd_delay > 1)
        bprintf(b,
                ("NB: It can take up to %lu seconds for password changes "
                 "to take effect" CRLF), (config->accountd_passwd_delay));

    bputs(b, "<hr>" CRLF);

    html_form_start(session, b, "POST", "passwd");
    bputs(b, "<table>" CRLF);
    html_form_table_passwd_field(b, "Current Password:", "old", old, 20);
    html_form_table_passwd_field(b, "New Password:", "new", new, 20);
    html_form_table_passwd_field(b, "Confirm New Password:", "new2", new2,
                                 20);
    bputs(b, "</table>" CRLF);

    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Cancel");
    html_form_table_submit(b, "sub_change", "Change Password");
    bputs(b, "</tr>" CRLF);

    bputs(b, "</table>" CRLF);
    html_form_end(b);
    html_end(b);

    response_html(request, 200);        /* Success */
}

/* ====================================================================== */

static BOOL check_passwd(char *password)
{
    return (T);
}

static BOOL
process_form(struct session *session, char *old, char *new, char *new2)
{
    struct account *account = session->account;
    struct request *request = session->request;
    struct pool *pool = request->pool;

    if (!(old && old[0])) {
        session_message(session, "Existing password not supplied");
        return (NIL);
    }

    if (!(new && new[0])) {
        session_message(session, "New password not supplied");
        return (NIL);
    }

    if (!(new2 && new2[0])) {
        session_message(session, "(Confirmed) New password not supplied");
        return (NIL);
    }

    if (strcmp(new, new2) != NIL) {
        session_message(session, "New passwords do not match");
        return (NIL);
    }

    if (!check_passwd(new)) {
        session_message(session, "Invalid Password: see notes below");
        return (NIL);
    }

    if (!account_change_password(session->account, pool, old, new)) {
        session_message(session, "%s", account_fetch_message(account));
        return (NIL);
    }

    return (T);
}

void cmd_passwd(struct session *session)
{
    struct request *request = session->request;
    char *old, *new, *new2;

    if (request->method == POST) {
        struct assoc *h;

        request_decode_post(request);
        h = request->form;

        if (assoc_lookup(h, "sub_change")) {
            if ((old = assoc_lookup(h, "old")))
                old = string_trim_whitespace(old);

            if ((new = assoc_lookup(h, "new")))
                new = string_trim_whitespace(new);

            if ((new2 = assoc_lookup(h, "new2")))
                new2 = string_trim_whitespace(new2);

            if (!process_form(session, old, new, new2)) {
                generate_form(session, old, new, new2);
                return;
            }
        }

        session_redirect(session, request, "manage");
        return;
    }

    generate_form(session, NIL, NIL, NIL);
}

==> prayer-1.0.16.tar.gz: prayer-1.0.16/prayer/cmd_passwd.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_passwd.c,v 1.7 2005/08/10 09:48:34 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void
html_form_table_passwd_field(struct buffer *b,
                             char *desc, char *name, char *value,
                             unsigned long size)
{
    bputs(b, "<tr>" CRLF);

    if (desc && desc[0])
        bprintf(b, "<td>%s</td>", desc);

    bprintf(b, "<td><input name=\"%s\" type=\"password\" value=\"", name);
    if (value)
        html_quote_string(b, value);
    bprintf(b, "\" size=\"%lu\"></td>" CRLF, size);

    bputs(b, "</tr>" CRLF);
}

static void
generate_form(struct session *session, char *old, char *new, char *new2)
{
    struct config *config = session->config;
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;

    /* Title bar */
    html_start(session, b, "Change Password");
    html_banner_toolbar(session, b, "passwd", 0);

    /* Title line */
    bprintf(b, "<h2 align=\"center\">Change Password</h2>" CRLF);

    bputs(b, "Restrictions on passwords:" CRLF);
    bputs(b, "<ul>" CRLF);
    bputs(b, "<li>Must be at least six characters in length</li>" CRLF);
    bputs(b,
          "<li>Must contain at least one letter and one non-letter</li>"
          CRLF);
    bputs(b, "</ul>" CRLF);
    if (config->accountd_passwd_delay > (2 * 60 * 60))
        bprintf(b,
                ("NB: It can take up to %lu hours for password changes "
                 "to take effect" CRLF),
                (config->accountd_passwd_delay / (60 * 60)));
    else if (config->accountd_passwd_delay > 120)
        bprintf(b,
                ("NB: It can take up to %lu minutes for password changes "
                 "to take effect" CRLF),
                (config->accountd_passwd_delay / 60));
    else if (config->accountd_passwd_delay > 1)
        bprintf(b,
                ("NB: It can take up to %lu seconds for password changes "
                 "to take effect" CRLF), (config->accountd_passwd_delay));

    bputs(b, "<hr>" CRLF);

    html_form_start(session, b, "POST", "passwd");
    bputs(b, "<table>" CRLF);
    html_form_table_passwd_field(b, "Current Password:", "old", old, 20);
    html_form_table_passwd_field(b, "New Password:", "new", new, 20);
    html_form_table_passwd_field(b, "Confirm New Password:", "new2", new2,
                                 20);
    bputs(b, "</table>" CRLF);

    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Cancel");
    html_form_table_submit(b, "sub_change", "Change Password");
    bputs(b, "</tr>" CRLF);

    bputs(b, "</table>" CRLF);
    html_form_end(b);
    html_end(b);

    response_html(request, 200);        /* Success */
}

/* ====================================================================== */

static BOOL check_passwd(char *password)
{
    return (T);
}

static BOOL
process_form(struct session *session, char *old, char *new, char *new2)
{
    struct account *account = session->account;
    struct request *request = session->request;
    struct pool *pool = request->pool;

    if (!(old && old[0])) {
        session_message(session, "Existing password not supplied");
        return (NIL);
    }

    if (!(new && new[0])) {
        session_message(session, "New password not supplied");
        return (NIL);
    }

    if (!(new2 && new2[0])) {
        session_message(session, "(Confirmed) New password not supplied");
        return (NIL);
    }

    if (strcmp(new, new2) != NIL) {
        session_message(session, "New passwords do not match");
        return (NIL);
    }

    if (!check_passwd(new)) {
        session_message(session, "Invalid Password: see notes below");
        return (NIL);
    }

    if (!account_change_password(session->account, pool, old, new)) {
        char *msg = account_fetch_message(account);

        session_message(session, "Failed to change password: %s", msg);
        session_log(session,
                    "[cmd_passwd] Failed to change password: %s", msg);
        return (NIL);
    }

    session_message(session, "Password update successful");
    return (T);
}

void cmd_passwd(struct session *session)
{
    struct request *request = session->request;
    char *old, *new, *new2;

    if (request->method == POST) {
        struct assoc *h;

        request_decode_post(request);
        h = request->form;

        if (assoc_lookup(h, "sub_change")) {
            if ((old = assoc_lookup(h, "old")))
                old = string_trim_whitespace(old);

            if ((new = assoc_lookup(h, "new")))
                new = string_trim_whitespace(new);

            if ((new2 = assoc_lookup(h, "new2")))
                new2 = string_trim_whitespace(new2);

            if (!process_form(session, old, new, new2)) {
                generate_form(session, old, new, new2);
                return;
            }
        }

        session_redirect(session, request, "manage");
        return;
    }

    generate_form(session, NIL, NIL, NIL);
}
